@@ -7,8 +7,6 @@
  * This license is available at: http://opensource.org/licenses/Apache-2.0.
  */
 
-#include <stdarg.h>
-
 #include "ac.h"
 #include "bbox.h"
 #include "charpath.h"
@@ -18,7 +16,6 @@
 #define DONT_COMBINE_PATHS 1
 
 #define DMIN 50       /* device minimum (one-half of a device pixel) */
-#define GROWBUFF 2048 /* Amount to grow output buffer, if necessary. */
 #define FONTSTKLIMIT 22
 
 #define MAINHINTS -1
@@ -35,10 +32,9 @@
 
 static bool firstMT;
 static Cd* refPtArray = NULL;
-static char* outbuff;
-static int16_t masterCount;
+static ACBuffer* outbuff;
+static int masterCount;
 static const char** masterNames;
-static size_t byteCount, buffSize;
 static PathList* pathlist = NULL;
 static indx hintsMasterIx = 0; /* The index of the master we read hints from */
 
@@ -48,32 +44,11 @@ static int16_t GetOperandCount(int16_t);
 static void GetLengthandSubrIx(int16_t, int16_t*, int16_t*);
 
 /* macros */
+#define WriteToBuffer(...) ACBufferWriteF(outbuff, __VA_ARGS__)
 #define WRTNUM(i) WriteToBuffer("%d ", (int)(i))
-#define WRTNUMA(i) WriteToBuffer("%0.2f ", roundf((float)(i)*100) / 100)
-#define WriteStr(str) WriteToBuffer("%s ", str)
+#define WRTNUMA(i) WriteToBuffer("%0.2f ", round((double)(i)*100) / 100)
 #define WriteSubr(val) WriteToBuffer("%d subr ", val)
 
-/* Checks if buffer needs to grow before writing out string. */
-static void
-WriteToBuffer(char* format, ...)
-{
-    char outstr[MAXBUFFLEN + 1];
-    size_t len;
-    va_list va;
-
-    va_start(va, format);
-    len = vsnprintf(outstr, MAXBUFFLEN, format, va);
-    va_end(va);
-
-    if ((byteCount + len) > buffSize) {
-        buffSize += GROWBUFF;
-        outbuff = (char*)ReallocateMem(outbuff, buffSize, "file buffer");
-    }
-
-    sprintf(outbuff + byteCount, "%s", outstr);
-    byteCount += len;
-}
-
 static void
 WriteX(Fixed x)
 {
@@ -116,31 +91,6 @@ GetCPIx(indx mIx, int32_t pathIx)
     return (-1);
 }
 
-/* Locates the first MT preceding the given path element. */
-static int
-GetMTIx(indx mIx, indx pathIx)
-{
-    indx ix;
-
-    for (ix = pathIx; ix >= 0; ix--)
-        if (pathlist[mIx].path[ix].type == RMT)
-            return ix;
-    LogMsg(LOGERROR, NONFATALERROR, "No moveto.");
-    return (-1);
-}
-
-/* Locates the first MT SUCCEEDING the given path element. */
-static int
-GetNextMTIx(indx mIx, indx pathIx)
-{
-    indx ix;
-
-    for (ix = pathIx; ix < gPathEntries; ix++)
-        if (pathlist[mIx].path[ix].type == RMT)
-            return ix;
-    return (-1);
-}
-
 static void
 GetEndPoint1(indx mIx, int32_t pathIx, Fixed* ptX, Fixed* ptY)
 {
@@ -203,7 +153,11 @@ GetCoordFromType(int16_t pathtype, Cd* coord, indx mIx, indx eltno)
             GetCoordFromType(pathlist[mIx].path[eltno - 1].type, coord, mIx,
                              eltno - 1);
             break;
-    };
+        default:
+            LogMsg(LOGERROR, NONFATALERROR, "Unrecognized path type");
+            memset(coord, 0, sizeof(Cd));
+            break;
+    }
 }
 
 static const char* const pathTypes[] = { "moveto", "lineto", "curveto",
@@ -318,119 +272,11 @@ ChangetoCurve(indx mIx, indx pathIx)
     return true;
 }
 
-static bool
-ZeroLengthCP(indx mIx, indx pathIx)
-{
-    Cd startPt = { 0, 0 }, endPt = { 0, 0 };
-
-    GetEndPoints1(mIx, pathIx, &startPt, &endPt);
-    return (startPt.x == endPt.x && startPt.y == endPt.y);
-}
-
-/* Subtracts or adds one unit from the segment at pathIx. */
-static void
-AddLine(indx mIx, indx pathIx)
-{
-    Fixed fixTwo = FixInt(2);
-    Fixed xoffset = 0, yoffset = 0;
-    Fixed xoffsetr = 0, yoffsetr = 0;
-    GlyphPathElt *start, *end, *thisone;
-    indx i, n;
-
-    if (pathlist[mIx].path[pathIx].type != RCT) {
-        LogMsg(WARNING, OK,
-               "Please convert the point closepath in master: "
-               "%s, to a line closepath.",
-               masterNames[mIx]);
-        return;
-    }
-    i = GetMTIx(mIx, pathIx) + 1;
-    start = &pathlist[mIx].path[i];
-    end = &pathlist[mIx].path[pathIx];
-    /* Check control points to find out if x or y value should be adjusted
-     in order to get a smooth curve. */
-    switch (start->type) {
-        case RDT:
-            LogMsg(WARNING, OK,
-                   "Please convert the point closepath to a line "
-                   "closepath in master: %s.",
-                   masterNames[mIx]);
-            return;
-        case RCT:
-            if ((abs(start->x1 - end->x2) < fixTwo) &&
-                (abs(start->x1 - pathlist[mIx].path[i - 1].x) < fixTwo))
-                yoffset = (start->y1 < end->y2 && end->y2 > 0) ||
-                              (start->y1 > end->y2 && end->y2 < 0)
-                            ? FixOne
-                            : -FixOne;
-            else if ((abs(start->y1 - end->y2) < fixTwo) &&
-                     (abs(start->y1 - pathlist[mIx].path[i - 1].y) < fixTwo))
-                xoffset = (start->x1 < end->x2 && end->x2 > 0) ||
-                              (start->x1 > end->x2 && end->x2 < 0)
-                            ? FixOne
-                            : -FixOne;
-            else {
-                LogMsg(WARNING, OK,
-                       "Could not modify point closepath in master "
-                       "'%s', near (%d, %d).",
-                       masterNames[mIx], FTrunc(end->x), FTrunc(end->y));
-                return;
-            }
-            break;
-        default:
-            LogMsg(LOGERROR, NONFATALERROR, "Bad description in master %s.",
-                   masterNames[mIx]);
-    }
-
-    thisone = &(pathlist[mIx].path[pathIx]);
-    thisone->x3 += xoffset;
-    xoffsetr = (xoffset == 0) ? 0 : ((thisone->rx3 < 0) ? FixOne : -FixOne);
-    thisone->rx3 += xoffsetr;
-
-    thisone->y3 += yoffset;
-    yoffsetr = (yoffset == 0) ? 0 : ((thisone->ry3 < 0) ? FixOne : -FixOne);
-    thisone->ry3 += yoffsetr;
-
-    /* Now, fix up the following MT's rx1, ry1 values
-     This fixes a LOOOONG-standing bug.    renner Wed Jul 16 09:33:50 1997*/
-    if ((n = GetNextMTIx(mIx, pathIx)) > 0) {
-        GlyphPathElt* nxtone = &(pathlist[mIx].path[n]);
-        nxtone->rx += (-xoffsetr);
-        nxtone->ry += (-yoffsetr);
-    }
-}
-
-/*
- Look for zero length closepaths.  If true then add a small
- one unit line to each design.  Because blending can cause
- small arithmetic errors at each point in the path the
- accumulation of such errors might cause the path to not end
- up at the same point where it started.  This can cause a sharp
- angle segment that may cause spike problems in old rasterizers,
- and in the qreducer (used for filling very large size glyphs).
- The one unit line, which is drawn by the closepath, takes up the
- slack for the arithmetic errors and avoids the spike problem.
- */
-static void
-CheckForZeroLengthCP(void)
-{
-    indx ix, pathIx;
-
-    for (ix = 0; ix < masterCount; ix++) {
-        for (pathIx = 0; pathIx < gPathEntries; pathIx++) {
-            if (pathlist[ix].path[pathIx].type == CP &&
-                ZeroLengthCP(ix, pathIx)) {
-                AddLine(ix, pathIx - 1);
-            }
-        }
-    }
-}
-
 /* Checks that glyph paths for multiple masters have the same
  number of points and in the same path order.  If this isn't the
  case the glyph is not included in the font. */
 static bool
-CompareGlyphPaths(const ACFontInfo* fontinfo, const char** glyphs)
+CompareGlyphPaths(const char** glyphs)
 {
     indx mIx, ix, i;
     int32_t totalPathElt, minPathLen;
@@ -450,11 +296,11 @@ CompareGlyphPaths(const ACFontInfo* fontinfo, const char** glyphs)
 
         if (hintsMasterIx == mIx) {
             /* read char data and hints from bez file */
-            if (!ReadGlyph(fontinfo, glyphs[mIx], true, gAddHints))
+            if (!ReadGlyph(glyphs[mIx], true, gAddHints))
                 return false;
         } else {
             /* read char data only */
-            if (!ReadGlyph(fontinfo, glyphs[mIx], true, false))
+            if (!ReadGlyph(glyphs[mIx], true, false))
                 return false;
         }
 
@@ -591,6 +437,7 @@ CurveBBox(indx mIx, int16_t hinttype, int32_t pathIx, Fixed* value)
             break;
         default:
             LogMsg(LOGERROR, NONFATALERROR, "Illegal hint type.");
+            return false;
     }
     if (p1 - maxval >= FixOne || p2 - maxval >= FixOne ||
         p1 - minval <= FixOne || p2 - minval <= FixOne) {
@@ -1040,129 +887,6 @@ Ct(Cd coord1, Cd coord2, Cd coord3, indx startix, int16_t length)
         }
 }
 
-static void
-ReadHorVStem3Values(indx pathIx, int16_t eltno, int16_t hinttype,
-                    bool* errormsg)
-{
-    indx ix;
-    HintElt** hintElt = NULL;
-    int16_t count;
-    bool ok = true;
-    Fixed min, dmin, mid, dmid, max, dmax;
-
-    for (ix = 0; ix < masterCount; ix++) {
-        count = 1;
-        if (ix == hintsMasterIx)
-            continue;
-        hintElt = (pathIx == MAINHINTS ? &pathlist[ix].mainhints
-                                       : &pathlist[ix].path[pathIx].hints);
-        /* Find specified hint element. */
-        while (*hintElt != NULL && count != eltno) {
-            hintElt = &(*hintElt)->next;
-            count++;
-        }
-        /* Check that RM or RV type is in pairs of threes. */
-        if (*hintElt == NULL || (*hintElt)->next == NULL ||
-            (*hintElt)->next->next == NULL) {
-            LogMsg(LOGERROR, NONFATALERROR,
-                   "Invalid format for hint operator: hstem3 or "
-                   "vstem3 in master: %s.",
-                   masterNames[ix]);
-        }
-        if ((*hintElt)->type != hinttype ||
-            (*hintElt)->next->type != hinttype ||
-            (*hintElt)->next->next->type != hinttype) {
-            LogMsg(LOGERROR, NONFATALERROR,
-                   "Invalid format for hint operator: hstem3 or "
-                   "vstem3 in master: %s.",
-                   masterNames[ix]);
-        }
-        min = (*hintElt)->leftorbot;
-        dmin = (*hintElt)->rightortop - min;
-        mid = (*hintElt)->next->leftorbot;
-        dmid = (*hintElt)->next->rightortop - mid;
-        max = (*hintElt)->next->next->leftorbot;
-        dmax = (*hintElt)->next->next->rightortop - max;
-        /* Check that counters are the same width and stems are the same width.
-         */
-        if (dmin != dmax || (((mid + dmid / 2) - (min + dmin / 2)) !=
-                             ((max + dmax / 2) - (mid + dmid / 2)))) {
-            ok = false;
-            break;
-        }
-    }
-    if (!ok) { /* change RM's to RY's or RV's to RB's for this element in each
-                  master */
-        int16_t newhinttype = (hinttype == (RM + ESCVAL) ? RY : RB);
-        if (*errormsg) {
-            LogMsg(INFO, OK,
-                   "Near miss for using operator: %s in master '%s'. "
-                   "(min=%d..%d[delta=%d], mid=%d..%d[delta=%d], "
-                   "max=%d..%d[delta=%d])",
-                   (hinttype == (RM + ESCVAL)) ? "vstem3" : "hstem3",
-                   masterNames[ix], FTrunc((*hintElt)->leftorbot),
-                   FTrunc((*hintElt)->rightortop),
-                   FTrunc((*hintElt)->rightortop - (*hintElt)->leftorbot),
-                   FTrunc((*hintElt)->next->leftorbot),
-                   FTrunc((*hintElt)->next->rightortop),
-                   FTrunc((*hintElt)->next->rightortop -
-                          (*hintElt)->next->leftorbot),
-                   FTrunc((*hintElt)->next->next->leftorbot),
-                   FTrunc((*hintElt)->next->next->rightortop),
-                   FTrunc((*hintElt)->next->next->rightortop -
-                          (*hintElt)->next->next->leftorbot));
-            *errormsg = false;
-        }
-        for (ix = 0; ix < masterCount; ix++) {
-            count = 1;
-            hintElt = (pathIx == MAINHINTS ? &pathlist[ix].mainhints
-                                           : &pathlist[ix].path[pathIx].hints);
-            /* Find specified hint element. */
-            while (*hintElt != NULL && count != eltno) {
-                hintElt = &(*hintElt)->next;
-                count++;
-            }
-            /* Already checked that hintElt->next, etc. exists,
-             so don't need to do it again. */
-            (*hintElt)->type = newhinttype;
-            (*hintElt)->next->type = newhinttype;
-            (*hintElt)->next->next->type = newhinttype;
-        }
-    }
-}
-
-/* Go through each hint element and check that all rm's and rv's
- meet the necessary criteria. */
-static void
-FindHandVStem3(HintElt** hintElt, indx pathIx, bool* errormsg)
-{
-    int16_t count = 1;
-
-    while (*hintElt != NULL) {
-        if ((*hintElt)->type == (RM + ESCVAL) ||
-            (*hintElt)->type == (RV + ESCVAL)) {
-            ReadHorVStem3Values(pathIx, count, (*hintElt)->type, errormsg);
-            /* RM's and RV's are in pairs of 3 */
-            hintElt = &(*hintElt)->next->next->next;
-            count += 3;
-        } else {
-            hintElt = &(*hintElt)->next;
-            count++;
-        }
-    }
-}
-
-static void
-CheckHandVStem3(void)
-{
-    indx ix;
-    bool errormsg = true;
-
-    FindHandVStem3(&pathlist[hintsMasterIx].mainhints, MAINHINTS, &errormsg);
-    for (ix = 0; ix < gPathEntries; ix++)
-        FindHandVStem3(&pathlist[hintsMasterIx].path[ix].hints, ix, &errormsg);
-}
-
 static void
 CheckFlexValues(int16_t* operator, indx eltix, indx flexix, bool* xequal,
                 bool* yequal)
@@ -1445,6 +1169,8 @@ WriteUnmergedHints(indx pathEltIx, indx mIx)
     UnallocateMem(hintList);
 }
 
+#define BREAK_ON_NULL_HINTARRAY_IX if(hintArray[ix] == NULL) break
+
 static void
 WriteHints(indx pathEltIx)
 {
@@ -1470,6 +1196,7 @@ WriteHints(indx pathEltIx)
         indx startix = 0;
         int16_t hinttype = hintArray[hintsMasterIx]->type;
         for (ix = 0; ix < masterCount; ix++) {
+            BREAK_ON_NULL_HINTARRAY_IX;
             hintArray[ix]->rightortop -=
               hintArray[ix]->leftorbot; /* relativize */
             if ((hinttype == RY || hinttype == (RM + ESCVAL)))
@@ -1481,6 +1208,7 @@ WriteHints(indx pathEltIx)
         }
         lbsame = rtsame = true;
         for (ix = 1; ix < masterCount; ix++) {
+            BREAK_ON_NULL_HINTARRAY_IX;
             if (hintArray[ix]->leftorbot != hintArray[ix - 1]->leftorbot)
                 lbsame = false;
             if (hintArray[ix]->rightortop != hintArray[ix - 1]->rightortop)
@@ -1491,17 +1219,21 @@ WriteHints(indx pathEltIx)
             WriteOneHintVal(hintArray[0]->rightortop);
         } else if (lbsame) {
             WriteOneHintVal(hintArray[0]->leftorbot);
-            for (ix = 0; ix < masterCount; ix++)
+            for (ix = 0; ix < masterCount; ix++) {
+                BREAK_ON_NULL_HINTARRAY_IX;
                 WriteOneHintVal((ix == 0 ? hintArray[ix]->rightortop
                                          : hintArray[ix]->rightortop -
                                              hintArray[0]->rightortop));
+            }
             GetLengthandSubrIx(1, &length, &subrIx);
             WriteSubr(subrIx);
         } else if (rtsame) {
-            for (ix = 0; ix < masterCount; ix++)
+            for (ix = 0; ix < masterCount; ix++) {
+                BREAK_ON_NULL_HINTARRAY_IX;
                 WriteOneHintVal((ix == 0 ? hintArray[ix]->leftorbot
                                          : hintArray[ix]->leftorbot -
                                              hintArray[0]->leftorbot));
+            }
             GetLengthandSubrIx(1, &length, &subrIx);
             WriteSubr(subrIx);
             WriteOneHintVal(hintArray[0]->rightortop);
@@ -1515,6 +1247,7 @@ WriteHints(indx pathEltIx)
             }
             for (opix = 0; opix < opcount; opix += length) {
                 for (ix = startix; ix < masterCount; ix++) {
+                    BREAK_ON_NULL_HINTARRAY_IX;
                     if (opix == 0)
                         WriteOneHintVal((ix == 0 ? hintArray[ix]->leftorbot
                                                  : hintArray[ix]->leftorbot -
@@ -1718,7 +1451,7 @@ SamePathValues(indx eltIx, int16_t op, indx startIx, int16_t length)
  combines them into a single path description using new subroutine
  calls 7 - 11. */
 static void
-WritePaths(char** outBuffers, size_t* outLengths)
+WritePaths(ACBuffer** outBuffers)
 {
     indx ix, eltix, opix, startIx, mIx;
     int16_t length, subrIx, opcount, op;
@@ -1728,8 +1461,6 @@ WritePaths(char** outBuffers, size_t* outLengths)
     for (mIx = 0; mIx < masterCount; mIx++) {
         PathList path = pathlist[mIx];
 
-        byteCount = 0;
-        buffSize = outLengths[mIx];
         outbuff = outBuffers[mIx];
 
         WriteToBuffer("%% %s\n", gGlyphName);
@@ -1759,13 +1490,9 @@ WritePaths(char** outBuffers, size_t* outLengths)
 
             WritePathElt(mIx, eltix, op, 0, opcount);
 
-            WriteToBuffer(GetOperator(op));
-            WriteToBuffer("\n");
+            WriteToBuffer("%s\n", GetOperator(op));
         }
         WriteToBuffer("ed\n");
-
-        outLengths[mIx] = byteCount;
-        outBuffers[mIx] = outbuff;
     }
     return;
 #endif /* DONT_COMBINE_PATHS */
@@ -1857,8 +1584,7 @@ WritePaths(char** outBuffers, size_t* outLengths)
                 if (subrIx >= 0 && op != CP)
                     WriteSubr(subrIx);
             } /* end of for opix */
-        WriteStr(GetOperator(op));
-        WriteToBuffer("\n");
+        WriteToBuffer("%s\n", GetOperator(op));
     } /* end of for eltix */
     WriteToBuffer("ed\n");
 }
@@ -1975,9 +1701,8 @@ GetLengthandSubrIx(int16_t opcount, int16_t* length, int16_t* subrIx)
  *************/
 
 bool
-MergeGlyphPaths(const ACFontInfo* fontinfo, const char** srcglyphs,
-                int nmasters, const char** masters, char** outbuffers,
-                size_t* outlengths)
+MergeGlyphPaths(const char** srcglyphs, int nmasters, const char** masters,
+                ACBuffer** outbuffers)
 {
     bool ok;
     /* This requires that  master  hintsMasterIx has already been hinted with
@@ -1985,18 +1710,16 @@ MergeGlyphPaths(const ACFontInfo* fontinfo, const char** srcglyphs,
     masterCount = nmasters;
     masterNames = masters;
 
-    ok = CompareGlyphPaths(fontinfo, srcglyphs);
+    ok = CompareGlyphPaths(srcglyphs);
     if (ok) {
-        CheckForZeroLengthCP();
         SetSbandWidth();
         if (gAddHints && hintsMasterIx >= 0 && gPathEntries > 0) {
             if (!ReadandAssignHints()) {
                 LogMsg(LOGERROR, FATALERROR,
                        "Path problem in ReadAndAssignHints");
             }
-            CheckHandVStem3();
         }
-        WritePaths(outbuffers, outlengths);
+        WritePaths(outbuffers);
     }
     FreePathElements(masterCount);
 
